﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace Calendar
{
    public partial class MainPage : UserControl
    {

        List<TextBlock> calendarTexts = new List<TextBlock>();
        List<Rectangle> calendarRectangles = new List<Rectangle>();

        int year, month;

        public MainPage()
        {
            InitializeComponent();
            MakeShapes();
            year = DateTime.Now.Year;
            month = DateTime.Now.Month;
            DispCalendar(year, month);
        }

        private void MakeShapes()
        {
            for (int i = 0; i < 37; i++)
            {
                int r = i / 7 + 1;
                int c = i % 7;
                Rectangle rect = new Rectangle();
                Grid.SetColumn(rect, c);
                Grid.SetRow(rect, r);

                TextBlock text = new TextBlock();
                Grid.SetColumn(text, c);
                Grid.SetRow(text, r);

                if (i < 35)
                    gridMain.Children.Add(rect);

                calendarRectangles.Add(rect);
                gridMain.Children.Add(text);
                calendarTexts.Add(text);
            }
        }

        void DispCalendar(int y, int m)
        {
            DateTime firstMonth = new DateTime(y, m, 1);
            DateTime currentDate =
                firstMonth.AddDays(-(int)(firstMonth.DayOfWeek));
            textBlock1.Text = string.Format("{0:yyyy年MM月}", firstMonth);

            for (int i = 0; i < calendarTexts.Count; i++)
            {
                string rectStyle, textStyle;

                if (m == currentDate.Month)
                {
                    // 今月分
                    rectStyle = IsSun(currentDate) ? "DateSunRectStyle" :
                        IsSat(currentDate) ? "DateSatRectStyle" : "DateDayRectStyle";
                    if (i < 35)
                        textStyle = IsSun(currentDate) ? "DateSunTextStyle" :
                            IsSat(currentDate) ? "DateSatTextStyle" : "DateDayTextStyle";
                    else
                        textStyle = IsSun(currentDate) ? "PackSunTextStyle" : "PackDayTextStyle";
                }
                else
                {
                    // 前月または翌月
                    rectStyle = IsSun(currentDate) ? "WhiteSunRectStyle" :
                        IsSat(currentDate) ? "WhiteSatRectStyle" : "WhiteDayRectStyle";
                    textStyle = IsSun(currentDate) ? "WhiteSunTextStyle" :
                        IsSat(currentDate) ? "WhiteSatTextStyle" : "WhiteDayTextStyle";
                }
                calendarRectangles[i].Style = this.Resources[rectStyle] as Style;
                calendarTexts[i].Style = this.Resources[textStyle] as Style;

                if (i < 35 || i > 34 && m == currentDate.Month)
                    calendarTexts[i].Text = currentDate.Day.ToString();
                else
                    calendarTexts[i].Text = "";

                currentDate = currentDate.AddDays(1);
            }
        }

        private bool IsSun(DateTime dt)
        {
            return (dt.DayOfWeek == DayOfWeek.Sunday);
        }

        private bool IsSat(DateTime dt)
        {
            return (dt.DayOfWeek == DayOfWeek.Saturday);
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            if (month == 1)
            {
                month = 12;
                year--;
            }
            else
                month--;

            DispCalendar(year, month);
        }

        private void button2_Click(object sender, RoutedEventArgs e)
        {
            if (month == 12)
            {
                month = 1;
                year++;
            }
            else
                month++;

            DispCalendar(year, month);
        }
    }
}
