﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;

namespace Scratch
{
    public partial class MainPage : UserControl
    {
        bool isDrawing;
        PointCollection linePoint;
        Point lastPoint;

        string kujiMessage;

        public MainPage()
        {
            InitializeComponent();
            InitScratch();
        }

        private void image_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            isDrawing = true;
            ((UIElement)sender).CaptureMouse();
            Point p = e.GetPosition(hitImage);
            if (PointAdd(p))
                MakeShape();
        }

        private void image_MouseMove(object sender, MouseEventArgs e)
        {
            if (isDrawing)
            {
                Point p = e.GetPosition(hitImage);
                if (PointAdd(p))
                    MakeShape();
            }
        }

        private void image_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (isDrawing)
            {
                MakeShape();
                CheckClear();
                ((UIElement)sender).ReleaseMouseCapture();
            }
            isDrawing = false;
        }

        private bool PointAdd(Point p)
        {
            if (p.X > -12 && p.X < hitImage.ActualWidth && p.Y > -12 &&
                p.Y < hitImage.ActualHeight &&
                Math.Sqrt(Math.Pow(p.X - lastPoint.X, 2) + Math.Pow(p.Y - lastPoint.Y, 2)) > 5)
            {
                linePoint.Add(p);
                lastPoint = p;
                return true;
            }
            else
                return false;
        }

        private void MakeShape()
        {
            hitImage.Clip = GetScratchGeometry();
        }

        private void CheckClear()
        {
            Path p = new Path();
            p.StrokeThickness = 1;
            p.Fill = new SolidColorBrush(Colors.Black);
            p.Data = GetScratchGeometry();

            Canvas c = new Canvas()
            {
                Width = hitImage.ActualWidth,
                Height = hitImage.ActualHeight,
                Children = {p}
            };
            WriteableBitmap w = new WriteableBitmap(c, null);

            double i = 0;
            int white = (255 << 24);
            for (int y = 30; y < 60; y++)
            {
                for (int x = 20; x < 70; x++)
                {
                    if (w.Pixels[x + y * w.PixelWidth] == white)
                        i++;
                }
            }

            double r = i / ((60 - 30) * (70 - 20));

            if (r > 0.5)
            {
                textBlock1.Text = kujiMessage;
                button1.IsEnabled = true;
            }
            else
                textBlock1.Text = "もう少し削ってください";
        }

        private Geometry GetScratchGeometry()
        {
            GeometryGroup g = new GeometryGroup();
            g.FillRule = FillRule.Nonzero;

            for (int i = 0; i < linePoint.Count; i++)
            {
                RectangleGeometry geo = new RectangleGeometry();
                geo.Rect = new Rect(linePoint[i], new Size(12, 12));
                geo.RadiusX = 4;
                geo.RadiusY = 4;
                g.Children.Add(geo);
            }
            return g;
        }

        void InitScratch()
        {
            linePoint = new PointCollection();
            MakeShape();
            button1.IsEnabled = false;
            Random myRandom = new Random();
            double d = myRandom.NextDouble();
            int s = 0;

            if (d < 0.8)
                s = 0;
            else if (d < 0.9)
                s = 1;
            else if (d < 0.95)
                s = 2;
            else
                s = 3;

            String[] imageName = new String[] {
                "kuji0.png", "kuji3.png", "kuji2.png", "kuji1.png" };
            String[] kuji = new String[] { "残念でした", "3等です", "2等です", "1等です" };
            kujiMessage = kuji[s];

            hitImage.Source = new BitmapImage(new Uri("Images/" + imageName[s], UriKind.Relative));
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            InitScratch();
        }
    }
}
