﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.Xml.Linq;
using System.IO;
using System.Web.Hosting;

namespace SilverQuiz.Web.ClientBin
{
    // メモ: [リファクター] メニューの [名前の変更] コマンドを使用すると、コード、svc、および config ファイルで同時にクラス名 "QuizService" を変更できます。
    public class QuizService : IQuizService
    {
        public QuizCategory[] GetQuizCategory()
        {
            string path = Path.Combine(HostingEnvironment.ApplicationPhysicalPath,
                 "QuizCategory.xml");

            StreamReader sr = new StreamReader(path);
            XElement quizXml = XElement.Load(sr);
            IEnumerable<QuizCategory> catList = from cl in quizXml.Elements("Category")
                                                select new QuizCategory(
                                                    (string)cl.Element("Title"),
                                                    (string)cl.Element("Id"));
            QuizCategory[] categories = catList.ToArray();
            return categories;
        }

        public Quiz[] GetQuiz(string code)
        {
            string path = Path.Combine(
                HostingEnvironment.ApplicationPhysicalPath,
                "Quiz.xml");

            StreamReader sr = new StreamReader(path);
            XElement quizXml = XElement.Load(sr);

            Random myRandom = new Random();
            // xmlからクイズをロードする
            IEnumerable<Quiz> quizList =
              (from cl in quizXml.Descendants("Quiz")
               where (cl.Parent.Name == "QuizGroup") &&
                     (cl.Parent.Attribute("Id").Value == code)
               orderby myRandom.Next()
               select new Quiz(
                   (string)cl.Element("Question"),
                   (string)cl.Element("Answer"),
                   (string[])((
                       from s in cl.Elements("Answer")
                       orderby myRandom.Next()
                       select s.Value).ToArray()),
                   (string)cl.Element("Comment"))).Take(10);

            Quiz[] quizArray = quizList.ToArray();

            // 問題に番号を付与する
            for (int i = 0; i < quizArray.GetLength(0); i++)
                quizArray[i].Number = i + 1;
            return quizArray;
        }
    }

    [DataContract]
    public class QuizCategory
    {
        [DataMember]
        public string Title { get; set; }
        [DataMember]
        public string Id { get; set; }

        public QuizCategory(string title, string id)
        {
            Title = title;
            Id = id;
        }
    }

    [DataContract]
    public class Quiz
    {
        [DataMember]
        public int Number { get; set; }
        [DataMember]
        public string Question { get; set; }
        [DataMember]
        public string[] Select { get; set; }
        [DataMember]
        public string Answer { get; set; }
        [DataMember]
        public string Comment { get; set; }

        public Quiz(string question, string correctAnswer, string[] selection, string comment)
        {
            Question = question;
            Answer = correctAnswer;
            Select = selection;
            Comment = comment;
        }
    }
}
