﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Navigation;
using System.Xml.Linq;
using SilverQuiz.QuizServiceReference;
using System.Windows.Browser;
using System.ServiceModel;

namespace SilverQuiz
{
    public partial class Page2 : Page
    {
        public List<Quiz> QuizList;
        public int CurrentIndex;
        static public int? QuestionCount;
        static public int? CorrectCount;

        public Page2()
        {
            InitializeComponent();
        }

        // ユーザーがこのページに移動したときに実行されます。
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            LayoutRoot.Visibility =
                System.Windows.Visibility.Collapsed;
            InitQuiz();
        }

        private void InitQuiz()
        {
            // 指定されたIDを取得する
            string id = this.NavigationContext.QueryString["id"];

            //　クイズをダウンロードする
            QuizServiceClient qscl = new QuizServiceClient();
            Uri documentUri = HtmlPage.Document.DocumentUri;
            string serviceUri = new Uri(documentUri, "QuizService.svc").AbsoluteUri;
            qscl.Endpoint.Address = new EndpointAddress(serviceUri);

            qscl.GetQuizCompleted += new EventHandler<GetQuizCompletedEventArgs>(qscl_GetQuizCompleted);
            qscl.GetQuizAsync(id);
        }

        void qscl_GetQuizCompleted(object sender, GetQuizCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                QuizList = e.Result.ToList();
                QuestionCount = QuizList.Count;
                CorrectCount = 0;
                if (QuestionCount == 0)
                    navigateTopPage();
                else
                {
                    // 最初の問題を表示する
                    LayoutRoot.DataContext = QuizList[CurrentIndex];
                    LayoutRoot.Visibility =
                        System.Windows.Visibility.Visible;
                }
            }
            else
                navigateTopPage();
        }

        private void selectButton_Click(object sender, RoutedEventArgs e)
        {
            string selected = (sender as Button).Content as string;

            Storyboard fadeInStoryboard;
            if (selected == QuizList[CurrentIndex].Answer)
            {
                CorrectCount++;
                fadeInStoryboard = CorrectPanelAnimation;
            }
            else
                fadeInStoryboard = IncorrectPanelAnimation;

            AnswerAnimation.Begin();
            fadeInStoryboard.Begin();
        }

        private void nextButton_Click(object sender, RoutedEventArgs e)
        {
            if (CurrentIndex < QuestionCount - 1)
            {
                CurrentIndex++;
                LayoutRoot.DataContext = QuizList[CurrentIndex];
                NextQuizAnimation.Begin();
            }
            else
            {
                string id = this.NavigationContext.QueryString["id"];
                this.NavigationService.Navigate(
                    new Uri("Result/" + id, UriKind.Relative));
            }
        }

        private void navigateTopPage()
        {
            this.NavigationService.Navigate(new Uri("", UriKind.Relative));
        }
    }
}
