﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace Suijaku
{

    [TemplateVisualState(Name = "Normal", GroupName = "CommonStates"),
    TemplateVisualState(Name = "MouseOver", GroupName = "CommonStates"),
    TemplateVisualState(Name = "Front", GroupName = "FlipStates"),
    TemplateVisualState(Name = "Back", GroupName = "FlipStates")]
    public class Flipcard : Control
    {
        public static readonly DependencyProperty FrontSourceProperty = DependencyProperty.Register("FrontSource", typeof(ImageSource), typeof(Flipcard), null);
        public static readonly DependencyProperty BuffFrontSourceProperty = DependencyProperty.Register("BuffFrontSource", typeof(ImageSource), typeof(Flipcard), new PropertyMetadata(OnBuffFrontSourceChanged));
        public static readonly DependencyProperty BackSourceProperty = DependencyProperty.Register("BackSource", typeof(ImageSource), typeof(Flipcard), null);
        public static readonly DependencyProperty ImageWidthProperty = DependencyProperty.Register("ImageWidth", typeof(double), typeof(Flipcard), null);
        public static readonly DependencyProperty ImageHeightProperty = DependencyProperty.Register("ImageHeight", typeof(double), typeof(Flipcard), null);
        public static readonly DependencyProperty IsFrontProperty = DependencyProperty.Register("IsFront", typeof(bool), typeof(Flipcard), new PropertyMetadata(OnIsFrontChanged));
        public static readonly DependencyProperty IsLockedProperty = DependencyProperty.Register("IsLocked", typeof(bool), typeof(Flipcard),  new PropertyMetadata(OnIsFrontChanged));
        public static readonly DependencyProperty CardIndexProperty = DependencyProperty.Register("CardIndex", typeof(int), typeof(Flipcard), null);

        private bool IsEntered;
        private ImageSource buffImageSource;

        public Flipcard()
        {
            this.DefaultStyleKey = typeof(Flipcard);
        }

        public ImageSource FrontSource
        {
            get
            {
                return (ImageSource)GetValue(FrontSourceProperty);
            }
            set
            {
                SetValue(FrontSourceProperty, value);
            }
        }

        public ImageSource BuffFrontSource
        {
            get
            {
                return (ImageSource)GetValue(BuffFrontSourceProperty);
            }
            set
            {
                SetValue(BuffFrontSourceProperty, value);
            }
        }

        public ImageSource BackSource
        {
            get
            {
                return (ImageSource)GetValue(BackSourceProperty);
            }
            set
            {
                SetValue(BackSourceProperty, value);
            }
        }

        public double ImageWidth
        {
            get
            {
                return (double)GetValue(ImageWidthProperty);
            }
            set
            {
                SetValue(ImageWidthProperty, value);
            }
        }

        public double ImageHeight
        {
            get
            {
                return (double)GetValue(ImageHeightProperty);
            }
            set
            {
                SetValue(ImageHeightProperty, value);
            }
        }

        public bool IsFront
        {
            get
            {
                return (bool)GetValue(IsFrontProperty);
            }
            set
            {
                SetValue(IsFrontProperty, value);
            }
        }

        public bool IsLocked
        {
            get
            {
                return (bool)GetValue(IsLockedProperty);
            }
            set
            {
                SetValue(IsLockedProperty, value);
            }
        }

        public int CardIndex
        {
            get
            {
                return (int)GetValue(CardIndexProperty);
            }
            set
            {
                SetValue(CardIndexProperty, value);
            }
        }

        private static void OnIsFrontChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            Flipcard s = d as Flipcard;
            s.ChangeVisualState(true);
        }

        private static void OnBuffFrontSourceChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            Flipcard s = d as Flipcard;
            if (!s.IsFront)
                s.FrontSource = e.NewValue as ImageSource;
            else
                s.buffImageSource = e.NewValue as ImageSource;
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            
            IsEntered = false;
            ChangeVisualState(false);

            VisualStateGroup g = (VisualStateGroup)(base.GetTemplateChild("FlipStates"));
            if (g != null)
            {
                g.CurrentStateChanged += FlipStates_CurrentStateChanged;
            }
        }

        public event RoutedEventHandler FrontChanged;
        public event RoutedEventHandler BackChanged;

        void FlipStates_CurrentStateChanged(object sender, VisualStateChangedEventArgs e)
        {
            if (e.NewState.Name == "Back")
            {
                if (buffImageSource != null)
                {
                    FrontSource = buffImageSource;
                    buffImageSource = null;
                }

                RoutedEventHandler handler = BackChanged;
                if (handler != null)
                    handler(this, null);
            }
            else
            {
                RoutedEventHandler handler = FrontChanged;
                if (handler != null)
                    handler(this, null);
            }
        }
        
        public void Flip()
        {
            IsFront = !IsFront;
        }

        private void ChangeVisualState(bool useTransition)
        {
            if (IsEntered && !IsLocked)
                VisualStateManager.GoToState(this, "MouseOver", useTransition);
            else
                VisualStateManager.GoToState(this, "Normal", useTransition);

            if (IsFront)
                VisualStateManager.GoToState(this, "Front", useTransition);
            else
                VisualStateManager.GoToState(this, "Back", useTransition);
        }

        protected override void OnMouseEnter(MouseEventArgs e)
        {
            base.OnMouseEnter(e);
            IsEntered = true;
            ChangeVisualState(true);
        }

        protected override void OnMouseLeave(MouseEventArgs e)
        {
            base.OnMouseLeave(e);
            IsEntered = false;
            ChangeVisualState(true);
        }

    }
}
