﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;

namespace Suijaku
{
    public partial class MainPage : UserControl
    {
        enum OpenMode
        {
            open1,
            open2,
            close,
            overopen
        }

        List<Flipcard> cards = new List<Flipcard>();
        Flipcard open1, open2, overcard;
        int cardCount, life;
        int maxLife = 6;

        public MainPage()
        {
            InitializeComponent();
            cardCount = 0;
            foreach (UIElement item in cardGrid.Children)
            {
                if (item is Flipcard)
                {
                    cards.Add(item as Flipcard);
                    item.MouseLeftButtonDown += card_MouseLeftButtonDown;
                    ((Flipcard)item).FrontChanged += card_FrontCompleted;
                    cardCount++;
                }
            }
            InitGame();
        }

        private void InitGame()
        {
            life = maxLife;
            AllLock(true);
            open1 = null;
            open2 = null;
            InitCard();
            foreach (Flipcard item in cards)
            {
                item.Visibility = Visibility.Visible;
                item.IsFront = false;
            }
            HeartDisplay(maxLife, life);
            AllLock(false);
        }

        private void card_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            Flipcard card = sender as Flipcard;

            if (card != null)
            {
                if (!card.IsFront && !card.IsLocked)
                {
                    AllLock(true);
                    if (open1 == null)
                        open1 = card;
                    else
                        open2 = card;
                    card.Flip();
                }
            }
        }

        private void card_FrontCompleted(object sender, EventArgs e)
        {
            if (sender == overcard)
            {
                MessageBox.Show("ゲームオーバーです");
                overcard = null;
            }

            if (open1 != null && open2 != null && sender == open2)
            {
                if (open1.CardIndex == open2.CardIndex)
                {
                    open1.Visibility = Visibility.Collapsed;
                    open2.Visibility = Visibility.Collapsed;
                    if (IsComplete())
                        MessageBox.Show("クリアおめでとう");
                    AllLock(false);
                }
                else
                {
                    if (life-- == 1)
                    {
                        AllLock(true);
                        foreach (Flipcard item in cards)
                        {
                            if (!item.IsFront && item.Visibility == Visibility.Visible)
                            {
                                overcard = item;
                                item.Flip();
                            }
                        }
                    }
                    else
                    {
                        // カードを閉じる
                        open1.Flip();
                        open2.Flip();
                        AllLock(false);
                    }
                    HeartDisplay(6, life);
                }
                open1 = null;
                open2 = null;
            }
            else if (open1 != null && open2 == null)
                AllLock(false);
        }

        private void AllLock(bool b)
        {
            foreach (Flipcard item in cards)
                item.IsLocked = b;
        }

        private bool IsComplete()
        {
            foreach (Flipcard item in cards)
            {
                if (item.Visibility == Visibility.Visible)
                    return false;
            }
            return true;
        }

        void InitCard()
        {
            int[] num = new int[cardCount];
            for (int i = 0; i < cardCount; i++)
                num[i] = i / 2;

            Random myRandom = new Random();
            for (int i = 0; i < cardCount; i++)
            {
                int k = num[i];
                int m = myRandom.Next(cardCount);
                num[i] = num[m];
                num[m] = k;
            }
            for (int i = 0; i < cardCount; i++)
            {
                cards[i].BuffFrontSource = new BitmapImage(new Uri("Images/Card" + (num[i] + 1).ToString() + ".png", UriKind.RelativeOrAbsolute));
                cards[i].CardIndex = num[i];
            }
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            InitGame();
        }

        private void HeartDisplay(int totalCount, int redCount)
        {
            lifeStackPanel.Children.Clear();
            for (int i = 0; i < totalCount; i++)
            {
                string heartName = i < redCount ? "Heart1.png" : "Heart2.png";
                BitmapImage b = new BitmapImage(new Uri("Images/" + heartName, UriKind.Relative));
                Image img = new Image() { Source = b, Stretch = Stretch.None };
                lifeStackPanel.Children.Add(img);
            }
        }
    }
}
